@@ -100,22 +100,32 @@ template<typename Config,
          typename CountsOutputIterator,
          typename RunsCountOutputIterator,
          typename LookbackScanState>
-ROCPRIM_KERNEL __launch_bounds__(device_params<Config>().kernel_config.block_size) void
-    non_trivial_kernel(const InputIterator           input,
-                       const OffsetsOutputIterator   offsets_output,
-                       const CountsOutputIterator    counts_output,
-                       const RunsCountOutputIterator runs_count_output,
-                       const LookbackScanState       scan_state,
-                       const std::size_t             grid_size,
-                       const std::size_t             size)
+inline hipError_t launch_non_trivial(detail::target_arch           arch,
+                                     const InputIterator           input,
+                                     const OffsetsOutputIterator   offsets_output,
+                                     const CountsOutputIterator    counts_output,
+                                     const RunsCountOutputIterator runs_count_output,
+                                     const LookbackScanState       scan_state,
+                                     const std::size_t             grid_size,
+                                     const std::size_t             size,
+                                     dim3                          grid,
+                                     dim3                          block,
+                                     size_t                        shmem,
+                                     hipStream_t                   stream)
 {
-    run_length_encode::non_trivial_kernel_impl<Config, OffsetCountPairType>(input,
-                                                                            offsets_output,
-                                                                            counts_output,
-                                                                            runs_count_output,
-                                                                            scan_state,
-                                                                            grid_size,
-                                                                            size);
+    auto kernel = [=](auto arch_config)
+    {
+        run_length_encode::non_trivial_kernel_impl<decltype(arch_config), OffsetCountPairType>(
+            input,
+            offsets_output,
+            counts_output,
+            runs_count_output,
+            scan_state,
+            grid_size,
+            size);
+    };
+
+    return execute_launch_plan<Config>(arch, kernel, grid, block, shmem, stream);
 }
 
 template<typename Config,
@@ -149,7 +159,7 @@ hipError_t run_length_encode_non_trivial_runs_impl(void*                   tempo
     detail::target_arch target_arch;
     ROCPRIM_RETURN_ON_ERROR(host_target_arch(stream, target_arch));
 
-    const non_trivial_runs_config_params params     = dispatch_target_arch<config>(target_arch);
+    const non_trivial_runs_config_params params = dispatch_target_arch<config, false>(target_arch);
     const unsigned int                   block_size = params.kernel_config.block_size;
     const unsigned int items_per_block = block_size * params.kernel_config.items_per_thread;
     const std::size_t  grid_size       = detail::ceiling_div(size, items_per_block);
@@ -177,11 +187,12 @@ hipError_t run_length_encode_non_trivial_runs_impl(void*                   tempo
 
     scan_state_type            scan_state{};
     scan_state_with_sleep_type scan_state_with_sleep{};
-    ROCPRIM_RETURN_ON_ERROR(scan_state_type::create(scan_state, scan_state_storage, grid_size, stream));
+    ROCPRIM_RETURN_ON_ERROR(
+        scan_state_type::create(scan_state, scan_state_storage, grid_size, stream));
     ROCPRIM_RETURN_ON_ERROR(scan_state_with_sleep_type::create(scan_state_with_sleep,
-                                                       scan_state_storage,
-                                                       grid_size,
-                                                       stream));
+                                                               scan_state_storage,
+                                                               grid_size,
+                                                               stream));
 
     auto with_scan_state
         = [use_sleep, scan_state, scan_state_with_sleep](auto&& func) mutable -> decltype(auto)
@@ -235,24 +246,23 @@ hipError_t run_length_encode_non_trivial_runs_impl(void*                   tempo
                                                 grid_size,
                                                 start);
 
-    with_scan_state(
+    ROCPRIM_RETURN_ON_ERROR(with_scan_state(
         [&](const auto scan_state)
         {
-            hipLaunchKernelGGL(
-                HIP_KERNEL_NAME(
-                    run_length_encode::non_trivial_kernel<config, offset_count_pair_type>),
-                dim3(grid_size),
-                dim3(block_size),
-                0,
-                stream,
+            return run_length_encode::launch_non_trivial<config, offset_count_pair_type>(
+                target_arch,
                 input + 0,
                 offsets_output,
                 counts_output,
                 runs_count_output,
                 scan_state,
                 grid_size,
-                size);
-        });
+                size,
+                dim3(grid_size),
+                dim3(block_size),
+                0,
+                stream);
+        }));
     ROCPRIM_DETAIL_HIP_SYNC_AND_RETURN_ON_ERROR("run_length_encode::non_trivial_kernel",
                                                 size,
                                                 start);
@@ -277,27 +287,27 @@ hipError_t run_length_encode_non_trivial_runs_impl(void*                   tempo
 /// * Ranges specified by \p unique_output and \p counts_output must have at least
 /// <tt>*runs_count_output</tt> (i.e. the number of runs) elements.
 ///
-/// \tparam Config - [optional] Configuration of the primitive, must be `default_config` or `run_length_encode_config`.
-/// \tparam InputIterator - random-access iterator type of the input range. Must meet the
+/// \tparam Config [optional] Configuration of the primitive, must be `default_config` or `run_length_encode_config`.
+/// \tparam InputIterator random-access iterator type of the input range. Must meet the
 /// requirements of a C++ InputIterator concept. It can be a simple pointer type.
-/// \tparam UniqueOutputIterator - random-access iterator type of the output range. Must meet the
+/// \tparam UniqueOutputIterator random-access iterator type of the output range. Must meet the
 /// requirements of a C++ OutputIterator concept. It can be a simple pointer type.
-/// \tparam CountsOutputIterator - random-access iterator type of the output range. Must meet the
+/// \tparam CountsOutputIterator random-access iterator type of the output range. Must meet the
 /// requirements of a C++ OutputIterator concept. It can be a simple pointer type.
-/// \tparam RunsCountOutputIterator - random-access iterator type of the output range. Must meet the
+/// \tparam RunsCountOutputIterator random-access iterator type of the output range. Must meet the
 /// requirements of a C++ OutputIterator concept. It can be a simple pointer type.
 ///
-/// \param [in] temporary_storage - pointer to a device-accessible temporary storage. When
+/// \param [in] temporary_storage pointer to a device-accessible temporary storage. When
 /// a null pointer is passed, the required allocation size (in bytes) is written to
 /// \p storage_size and function returns without performing the operation.
-/// \param [in,out] storage_size - reference to a size (in bytes) of \p temporary_storage.
-/// \param [in] input - iterator to the first element in the range of values.
-/// \param [in] size - number of element in the input range.
-/// \param [out] unique_output - iterator to the first element in the output range of unique values.
-/// \param [out] counts_output - iterator to the first element in the output range of lenghts.
-/// \param [out] runs_count_output - iterator to total number of runs.
-/// \param [in] stream - [optional] HIP stream object. Default is \p 0 (default stream).
-/// \param [in] debug_synchronous - [optional] If true, synchronization after every kernel
+/// \param [in,out] storage_size reference to a size (in bytes) of \p temporary_storage.
+/// \param [in] input iterator to the first element in the range of values.
+/// \param [in] size number of element in the input range.
+/// \param [out] unique_output iterator to the first element in the output range of unique values.
+/// \param [out] counts_output iterator to the first element in the output range of lenghts.
+/// \param [out] runs_count_output iterator to total number of runs.
+/// \param [in] stream [optional] HIP stream object. Default is \p 0 (default stream).
+/// \param [in] debug_synchronous [optional] If true, synchronization after every kernel
 /// launch is forced in order to check for errors. Default value is \p false.
 ///
 /// \returns \p hipSuccess (\p 0) after successful operation; otherwise a HIP runtime error of
@@ -390,27 +400,27 @@ inline hipError_t run_length_encode(void*                   temporary_storage,
 /// * Ranges specified by \p offsets_output and \p counts_output must have at least
 /// <tt>*runs_count_output</tt> (i.e. the number of non-trivial runs) elements.
 ///
-/// \tparam Config - [optional] Configuration of the primitive, must be `default_config` or `run_length_encode_config`.
-/// \tparam InputIterator - random-access iterator type of the input range. Must meet the
+/// \tparam Config [optional] Configuration of the primitive, must be `default_config` or `run_length_encode_config`.
+/// \tparam InputIterator random-access iterator type of the input range. Must meet the
 /// requirements of a C++ InputIterator concept. It can be a simple pointer type.
-/// \tparam OffsetsOutputIterator - random-access iterator type of the output range. Must meet the
+/// \tparam OffsetsOutputIterator random-access iterator type of the output range. Must meet the
 /// requirements of a C++ OutputIterator concept. It can be a simple pointer type.
-/// \tparam CountsOutputIterator - random-access iterator type of the output range. Must meet the
+/// \tparam CountsOutputIterator random-access iterator type of the output range. Must meet the
 /// requirements of a C++ OutputIterator concept. It can be a simple pointer type.
-/// \tparam RunsCountOutputIterator - random-access iterator type of the output range. Must meet the
+/// \tparam RunsCountOutputIterator random-access iterator type of the output range. Must meet the
 /// requirements of a C++ OutputIterator concept. It can be a simple pointer type.
 ///
-/// \param [in] temporary_storage - pointer to a device-accessible temporary storage. When
+/// \param [in] temporary_storage pointer to a device-accessible temporary storage. When
 /// a null pointer is passed, the required allocation size (in bytes) is written to
 /// \p storage_size and function returns without performing the operation.
-/// \param [in,out] storage_size - reference to a size (in bytes) of \p temporary_storage.
-/// \param [in] input - iterator to the first element in the range of values.
-/// \param [in] size - number of element in the input range.
-/// \param [out] offsets_output - iterator to the first element in the output range of offsets.
-/// \param [out] counts_output - iterator to the first element in the output range of lenghts.
-/// \param [out] runs_count_output - iterator to total number of runs.
-/// \param [in] stream - [optional] HIP stream object. Default is \p 0 (default stream).
-/// \param [in] debug_synchronous - [optional] If true, synchronization after every kernel
+/// \param [in,out] storage_size reference to a size (in bytes) of \p temporary_storage.
+/// \param [in] input iterator to the first element in the range of values.
+/// \param [in] size number of element in the input range.
+/// \param [out] offsets_output iterator to the first element in the output range of offsets.
+/// \param [out] counts_output iterator to the first element in the output range of lenghts.
+/// \param [out] runs_count_output iterator to total number of runs.
+/// \param [in] stream [optional] HIP stream object. Default is \p 0 (default stream).
+/// \param [in] debug_synchronous [optional] If true, synchronization after every kernel
 /// launch is forced in order to check for errors. Default value is \p false.
 ///
 /// \returns \p hipSuccess (\p 0) after successful operation; otherwise a HIP runtime error of
