@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,8 +21,8 @@
 #ifndef ROCPRIM_DEVICE_DEVICE_SCAN_BY_KEY_HPP_
 #define ROCPRIM_DEVICE_DEVICE_SCAN_BY_KEY_HPP_
 
-#include "../config.hpp"
 #include "../common.hpp"
+#include "../config.hpp"
 #include "../detail/temp_storage.hpp"
 #include "../detail/various.hpp"
 #include "../functional.hpp"
@@ -46,9 +46,9 @@ BEGIN_ROCPRIM_NAMESPACE
 namespace detail
 {
 
-template<lookback_scan_determinism Determinism,
+template<typename Config,
+         lookback_scan_determinism Determinism,
          bool                      Exclusive,
-         typename Config,
          typename KeyInputIterator,
          typename InputIterator,
          typename OutputIterator,
@@ -57,8 +57,9 @@ template<lookback_scan_determinism Determinism,
          typename BinaryFunction,
          typename LookbackScanState,
          typename AccType>
-void __global__ __launch_bounds__(device_params<Config>().kernel_config.block_size)
-    device_scan_by_key_kernel(const KeyInputIterator                       keys,
+inline hipError_t
+    launch_device_scan_by_key(detail::target_arch                          arch,
+                              const KeyInputIterator                       keys,
                               const InputIterator                          values,
                               const OutputIterator                         output,
                               const InitialValueType                       initial_value,
@@ -68,22 +69,31 @@ void __global__ __launch_bounds__(device_params<Config>().kernel_config.block_si
                               const size_t                                 size,
                               const size_t                                 starting_block,
                               const size_t                                 number_of_blocks,
-                              const ::rocprim::tuple<AccType, bool>* const previous_last_value)
+                              const ::rocprim::tuple<AccType, bool>* const previous_last_value,
+                              dim3                                         grid,
+                              dim3                                         block,
+                              size_t                                       shmem,
+                              hipStream_t                                  stream)
 {
-    device_scan_by_key_kernel_impl<Determinism, Exclusive, Config>(
-        keys,
-        values,
-        output,
-        static_cast<AccType>(get_input_value(initial_value)),
-        compare,
-        scan_op,
-        scan_state,
-        size,
-        starting_block,
-        number_of_blocks,
-        previous_last_value);
-}
 
+    auto kernel = [=](auto arch_config)
+    {
+        device_scan_by_key_kernel_impl<decltype(arch_config), Determinism, Exclusive>(
+            keys,
+            values,
+            output,
+            static_cast<AccType>(get_input_value(initial_value)),
+            compare,
+            scan_op,
+            scan_state,
+            size,
+            starting_block,
+            number_of_blocks,
+            previous_last_value);
+    };
+
+    return execute_launch_plan<Config>(arch, kernel, grid, block, shmem, stream);
+}
 
 template<lookback_scan_determinism Determinism,
          bool                      Exclusive,
@@ -117,7 +127,7 @@ inline hipError_t scan_by_key_impl(void* const           temporary_storage,
     {
         return result;
     }
-    const scan_by_key_config_params params = dispatch_target_arch<config>(target_arch);
+    const scan_by_key_config_params params = dispatch_target_arch<config, false>(target_arch);
 
     using wrapped_type = ::rocprim::tuple<AccType, bool>;
 
@@ -260,15 +270,11 @@ inline hipError_t scan_by_key_impl(void* const           temporary_storage,
         {
             start = std::chrono::steady_clock::now();
         }
-        with_scan_state(
+        ROCPRIM_RETURN_ON_ERROR(with_scan_state(
             [&](auto& scan_state)
             {
-                hipLaunchKernelGGL(
-                    HIP_KERNEL_NAME(device_scan_by_key_kernel<Determinism, Exclusive, config>),
-                    dim3(scan_blocks),
-                    dim3(block_size),
-                    0,
-                    stream,
+                return launch_device_scan_by_key<config, Determinism, Exclusive>(
+                    target_arch,
                     keys + offset,
                     input + offset,
                     output + offset,
@@ -279,8 +285,12 @@ inline hipError_t scan_by_key_impl(void* const           temporary_storage,
                     size,
                     i * number_of_blocks,
                     total_number_of_blocks,
-                    i > 0 ? as_const_ptr(previous_last_value) : nullptr);
-            });
+                    i > 0 ? as_const_ptr(previous_last_value) : nullptr,
+                    dim3(scan_blocks),
+                    dim3(block_size),
+                    0,
+                    stream);
+            }));
         ROCPRIM_DETAIL_HIP_SYNC_AND_RETURN_ON_ERROR("device_scan_by_key_kernel",
                                                     current_size,
                                                     start);
@@ -288,8 +298,7 @@ inline hipError_t scan_by_key_impl(void* const           temporary_storage,
     return hipSuccess;
 }
 
-
-}
+} // namespace detail
 
 /// \addtogroup devicemodule
 /// @{
@@ -313,41 +322,41 @@ inline hipError_t scan_by_key_impl(void* const           temporary_storage,
 /// * Ranges specified by \p keys_input, \p values_input, and \p values_output must have
 /// at least \p size elements.
 ///
-/// \tparam Config - [optional] Configuration of the primitive, must be `default_config` or `scan_by_key_config`.
-/// \tparam KeysInputIterator - random-access iterator type of the input range. It can be
+/// \tparam Config [optional] Configuration of the primitive, must be `default_config` or `scan_by_key_config`.
+/// \tparam KeysInputIterator random-access iterator type of the input range. It can be
 /// a simple pointer type.
-/// \tparam ValuesInputIterator - random-access iterator type of the input range. It can be
+/// \tparam ValuesInputIterator random-access iterator type of the input range. It can be
 /// a simple pointer type.
-/// \tparam ValuesOutputIterator - random-access iterator type of the output range. It can be
+/// \tparam ValuesOutputIterator random-access iterator type of the output range. It can be
 /// a simple pointer type.
-/// \tparam BinaryFunction - type of binary function used for scan. Default type
+/// \tparam BinaryFunction type of binary function used for scan. Default type
 /// is \p rocprim::plus<T>, where \p T is a \p value_type of \p InputIterator.
-/// \tparam KeyCompareFunction - type of binary function used to determine keys equality. Default type
+/// \tparam KeyCompareFunction type of binary function used to determine keys equality. Default type
 /// is \p rocprim::equal_to<T>, where \p T is a \p value_type of \p KeysInputIterator.
-/// \tparam AccType - accumulator type used to propagate the scanned values. Default type
+/// \tparam AccType accumulator type used to propagate the scanned values. Default type
 /// is value type of the input iterator.
 ///
-/// \param [in] temporary_storage - pointer to a device-accessible temporary storage. When
+/// \param [in] temporary_storage pointer to a device-accessible temporary storage. When
 /// a null pointer is passed, the required allocation size (in bytes) is written to
 /// \p storage_size and function returns without performing the scan operation.
-/// \param [in,out] storage_size - reference to a size (in bytes) of \p temporary_storage.
-/// \param [in] keys_input - iterator to the first element in the range of keys.
-/// \param [in] values_input - iterator to the first element in the range of values to scan.
-/// \param [out] values_output - iterator to the first element in the output value range.
-/// \param [in] size - number of element in the input range.
-/// \param [in] scan_op - binary operation function object that will be used for scanning
+/// \param [in,out] storage_size reference to a size (in bytes) of \p temporary_storage.
+/// \param [in] keys_input iterator to the first element in the range of keys.
+/// \param [in] values_input iterator to the first element in the range of values to scan.
+/// \param [out] values_output iterator to the first element in the output value range.
+/// \param [in] size number of element in the input range.
+/// \param [in] scan_op binary operation function object that will be used for scanning
 /// input values.
 /// The signature of the function should be equivalent to the following:
 /// <tt>T f(const T &a, const T &b);</tt>. The signature does not need to have
 /// <tt>const &</tt>, but function object must not modify the objects passed to it.
 /// Default is BinaryFunction().
-/// \param [in] key_compare_op - binary operation function object that will be used to determine keys equality.
+/// \param [in] key_compare_op binary operation function object that will be used to determine keys equality.
 /// The signature of the function should be equivalent to the following:
 /// <tt>bool f(const T &a, const T &b);</tt>. The signature does not need to have
 /// <tt>const &</tt>, but function object must not modify the objects passed to it.
 /// Default is KeyCompareFunction().
-/// \param [in] stream - [optional] HIP stream object. Default is \p 0 (default stream).
-/// \param [in] debug_synchronous - [optional] If true, synchronization after every kernel
+/// \param [in] stream [optional] HIP stream object. Default is \p 0 (default stream).
+/// \param [in] debug_synchronous [optional] If true, synchronization after every kernel
 /// launch is forced in order to check for errors. Default value is \p false.
 ///
 /// \returns \p hipSuccess (\p 0) after successful scan; otherwise a HIP runtime error of
@@ -505,44 +514,44 @@ inline hipError_t deterministic_inclusive_scan_by_key(void* const
 /// * Ranges specified by \p keys_input, \p values_input, and \p values_output must have
 /// at least \p size elements.
 ///
-/// \tparam Config - [optional] Configuration of the primitive, must be `default_config` or `scan_by_key_config`.
-/// \tparam KeysInputIterator - random-access iterator type of the input range. It can be
+/// \tparam Config [optional] Configuration of the primitive, must be `default_config` or `scan_by_key_config`.
+/// \tparam KeysInputIterator random-access iterator type of the input range. It can be
 /// a simple pointer type.
-/// \tparam ValuesInputIterator - random-access iterator type of the input range. It can be
+/// \tparam ValuesInputIterator random-access iterator type of the input range. It can be
 /// a simple pointer type.
-/// \tparam ValuesOutputIterator - random-access iterator type of the output range. It can be
+/// \tparam ValuesOutputIterator random-access iterator type of the output range. It can be
 /// a simple pointer type.
-/// \tparam InitValueType - type of the initial value.
-/// \tparam BinaryFunction - type of binary function used for scan. Default type
+/// \tparam InitValueType type of the initial value.
+/// \tparam BinaryFunction type of binary function used for scan. Default type
 /// is \p rocprim::plus<T>, where \p T is a \p value_type of \p InputIterator.
-/// \tparam KeyCompareFunction - type of binary function used to determine keys equality. Default type
+/// \tparam KeyCompareFunction type of binary function used to determine keys equality. Default type
 /// is \p rocprim::equal_to<T>, where \p T is a \p value_type of \p KeysInputIterator.
-/// \tparam AccType - accumulator type used to propagate the scanned values. Default type
+/// \tparam AccType accumulator type used to propagate the scanned values. Default type
 /// is 'InitValueType', unless it's 'rocprim::future_value'. Then it will be the wrapped input type.
 ///
-/// \param [in] temporary_storage - pointer to a device-accessible temporary storage. When
+/// \param [in] temporary_storage pointer to a device-accessible temporary storage. When
 /// a null pointer is passed, the required allocation size (in bytes) is written to
 /// \p storage_size and function returns without performing the scan operation.
-/// \param [in,out] storage_size - reference to a size (in bytes) of \p temporary_storage.
-/// \param [in] keys_input - iterator to the first element in the range of keys.
-/// \param [in] values_input - iterator to the first element in the range of values to scan.
-/// \param [out] values_output - iterator to the first element in the output value range.
-/// \param [in] initial_value - initial value to start the scan.
+/// \param [in,out] storage_size reference to a size (in bytes) of \p temporary_storage.
+/// \param [in] keys_input iterator to the first element in the range of keys.
+/// \param [in] values_input iterator to the first element in the range of values to scan.
+/// \param [out] values_output iterator to the first element in the output value range.
+/// \param [in] initial_value initial value to start the scan.
 /// A rocpim::future_value may be passed to use a value that will be later computed.
-/// \param [in] size - number of element in the input range.
-/// \param [in] scan_op - binary operation function object that will be used for scanning
+/// \param [in] size number of element in the input range.
+/// \param [in] scan_op binary operation function object that will be used for scanning
 /// input values.
 /// The signature of the function should be equivalent to the following:
 /// <tt>T f(const T &a, const T &b);</tt>. The signature does not need to have
 /// <tt>const &</tt>, but function object must not modify the objects passed to it.
 /// Default is BinaryFunction().
-/// \param [in] key_compare_op - binary operation function object that will be used to determine keys equality.
+/// \param [in] key_compare_op binary operation function object that will be used to determine keys equality.
 /// The signature of the function should be equivalent to the following:
 /// <tt>bool f(const T &a, const T &b);</tt>. The signature does not need to have
 /// <tt>const &</tt>, but function object must not modify the objects passed to it.
 /// Default is KeyCompareFunction().
-/// \param [in] stream - [optional] HIP stream object. Default is \p 0 (default stream).
-/// \param [in] debug_synchronous - [optional] If true, synchronization after every kernel
+/// \param [in] stream [optional] HIP stream object. Default is \p 0 (default stream).
+/// \param [in] debug_synchronous [optional] If true, synchronization after every kernel
 /// launch is forced in order to check for errors. Default value is \p false.
 ///
 /// \returns \p hipSuccess (\p 0) after successful scan; otherwise a HIP runtime error of
