@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,6 +20,13 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 
+#ifdef _CLANGD
+    // When using clangd, to allow the language server to function properly,
+    // some context of the template source is required to allow the language
+    // server to function properly.
+    #include "test_block_discontinuity.cpp.in"
+#endif
+
 test_suite_type_def(suite_name, name_suffix)
 
 typed_test_suite_def(RocprimBlockDiscontinuity, name_suffix, warp_params);
