@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -23,6 +23,28 @@
 #ifndef TEST_BLOCK_EXCHANGE_KERNELS_HPP_
 #define TEST_BLOCK_EXCHANGE_KERNELS_HPP_
 
+#include "../common_test_header.hpp"
+
+#include "../../common/utils_device_ptr.hpp"
+#include "test_utils.hpp"
+#include "test_utils_assertions.hpp"
+#include "test_utils_data_generation.hpp"
+#include "test_utils_types.hpp"
+
+#include <rocprim/block/block_exchange.hpp>
+#include <rocprim/block/block_load_func.hpp>
+#include <rocprim/block/block_store_func.hpp>
+#include <rocprim/config.hpp>
+#include <rocprim/device/config_types.hpp>
+#include <rocprim/type_traits.hpp>
+
+#include <algorithm>
+#include <cstddef>
+#include <numeric>
+#include <random>
+#include <type_traits>
+#include <vector>
+
 template<
     class Type,
     class OutputType,
@@ -198,15 +220,14 @@ auto test_block_exchange(int /*device_id*/) -> typename std::enable_if<Method ==
     // Generate data
     std::vector<type> input(size);
     std::vector<output_type> expected(size);
-    std::vector<output_type> output(size, (output_type)0);
 
     // Calculate input and expected results on host
     std::vector<type> values(size);
     test_utils::iota_modulo(values.begin(),
                             values.end(),
                             0,
-                            std::min<size_t>(test_utils::numeric_limits<type>::max(),
-                                             test_utils::numeric_limits<output_type>::max()));
+                            std::min<size_t>(rocprim::numeric_limits<type>::max(),
+                                             rocprim::numeric_limits<output_type>::max()));
     for(size_t bi = 0; bi < size / items_per_block; bi++)
     {
         for(size_t ti = 0; ti < block_size; ti++)
@@ -223,42 +244,26 @@ auto test_block_exchange(int /*device_id*/) -> typename std::enable_if<Method ==
     }
 
     // Preparing device
-    type* device_input;
-    HIP_CHECK(test_common_utils::hipMallocHelper(&device_input, input.size() * sizeof(typename decltype(input)::value_type)));
-    output_type* device_output;
-    HIP_CHECK(test_common_utils::hipMallocHelper(&device_output, output.size() * sizeof(typename decltype(output)::value_type)));
-
-    HIP_CHECK(
-        hipMemcpy(
-            device_input, input.data(),
-            input.size() * sizeof(type),
-            hipMemcpyHostToDevice
-        )
-    );
+    common::device_ptr<type>        device_input(input);
+    common::device_ptr<output_type> device_output(size);
 
     // Running kernel
     constexpr unsigned int grid_size = (size / items_per_block);
     hipLaunchKernelGGL(
-        HIP_KERNEL_NAME(blocked_to_striped_kernel<type, output_type, items_per_block, items_per_thread>),
-        dim3(grid_size), dim3(block_size), 0, 0,
-        device_input, device_output
-    );
+        HIP_KERNEL_NAME(
+            blocked_to_striped_kernel<type, output_type, items_per_block, items_per_thread>),
+        dim3(grid_size),
+        dim3(block_size),
+        0,
+        0,
+        device_input.get(),
+        device_output.get());
     HIP_CHECK(hipGetLastError());
     HIP_CHECK(hipDeviceSynchronize());
 
     // Reading results
-    HIP_CHECK(
-        hipMemcpy(
-            output.data(), device_output,
-            output.size() * sizeof(typename decltype(output)::value_type),
-            hipMemcpyDeviceToHost
-        )
-    );
-
+    const auto output = device_output.load();
     ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output, expected));
-
-    HIP_CHECK(hipFree(device_input));
-    HIP_CHECK(hipFree(device_output));
 }
 
 template<class T,
@@ -287,15 +292,14 @@ auto test_block_exchange(int /*device_id*/) -> typename std::enable_if<Method ==
     // Generate data
     std::vector<type> input(size);
     std::vector<output_type> expected(size);
-    std::vector<output_type> output(size, output_type(0));
 
     // Calculate input and expected results on host
     std::vector<type> values(size);
     test_utils::iota_modulo(values.begin(),
                             values.end(),
                             0,
-                            std::min<size_t>(test_utils::numeric_limits<type>::max(),
-                                             test_utils::numeric_limits<output_type>::max()));
+                            std::min<size_t>(rocprim::numeric_limits<type>::max(),
+                                             rocprim::numeric_limits<output_type>::max()));
     for(size_t bi = 0; bi < size / items_per_block; bi++)
     {
         for(size_t ti = 0; ti < block_size; ti++)
@@ -312,42 +316,26 @@ auto test_block_exchange(int /*device_id*/) -> typename std::enable_if<Method ==
     }
 
     // Preparing device
-    type* device_input;
-    HIP_CHECK(test_common_utils::hipMallocHelper(&device_input, input.size() * sizeof(typename decltype(input)::value_type)));
-    output_type* device_output;
-    HIP_CHECK(test_common_utils::hipMallocHelper(&device_output, output.size() * sizeof(typename decltype(output)::value_type)));
-
-    HIP_CHECK(
-        hipMemcpy(
-            device_input, input.data(),
-            input.size() * sizeof(type),
-            hipMemcpyHostToDevice
-        )
-    );
+    common::device_ptr<type>        device_input(input);
+    common::device_ptr<output_type> device_output(size);
 
     // Running kernel
     constexpr unsigned int grid_size = (size / items_per_block);
     hipLaunchKernelGGL(
-        HIP_KERNEL_NAME(striped_to_blocked_kernel<type, output_type, items_per_block, items_per_thread>),
-        dim3(grid_size), dim3(block_size), 0, 0,
-        device_input, device_output
-    );
+        HIP_KERNEL_NAME(
+            striped_to_blocked_kernel<type, output_type, items_per_block, items_per_thread>),
+        dim3(grid_size),
+        dim3(block_size),
+        0,
+        0,
+        device_input.get(),
+        device_output.get());
     HIP_CHECK(hipGetLastError());
     HIP_CHECK(hipDeviceSynchronize());
 
     // Reading results
-    HIP_CHECK(
-        hipMemcpy(
-            output.data(), device_output,
-            output.size() * sizeof(typename decltype(output)::value_type),
-            hipMemcpyDeviceToHost
-        )
-    );
-
+    const auto output = device_output.load();
     ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output, expected));
-
-    HIP_CHECK(hipFree(device_input));
-    HIP_CHECK(hipFree(device_output));
 }
 
 template<class T,
@@ -376,7 +364,6 @@ auto test_block_exchange(int device_id) -> typename std::enable_if<Method == 2>:
     // Generate data
     std::vector<type> input(size);
     std::vector<output_type> expected(size);
-    std::vector<output_type> output(size, output_type(0));
 
     unsigned int current_device_warp_size;
     HIP_CHECK(::rocprim::host_warp_size(device_id, current_device_warp_size));
@@ -390,8 +377,8 @@ auto test_block_exchange(int device_id) -> typename std::enable_if<Method == 2>:
     test_utils::iota_modulo(values.begin(),
                             values.end(),
                             0,
-                            std::min<size_t>(test_utils::numeric_limits<type>::max(),
-                                             test_utils::numeric_limits<output_type>::max()));
+                            std::min<size_t>(rocprim::numeric_limits<type>::max(),
+                                             rocprim::numeric_limits<output_type>::max()));
     for(size_t bi = 0; bi < size / items_per_block; bi++)
     {
         for(size_t wi = 0; wi < warps_no; wi++)
@@ -414,44 +401,26 @@ auto test_block_exchange(int device_id) -> typename std::enable_if<Method == 2>:
     }
 
     // Preparing device
-    type* device_input;
-    HIP_CHECK(test_common_utils::hipMallocHelper(&device_input, input.size() * sizeof(typename decltype(input)::value_type)));
-    output_type* device_output;
-    HIP_CHECK(test_common_utils::hipMallocHelper(&device_output, output.size() * sizeof(typename decltype(output)::value_type)));
-
-    HIP_CHECK(
-        hipMemcpy(
-            device_input, input.data(),
-            input.size() * sizeof(type),
-            hipMemcpyHostToDevice
-        )
-    );
+    common::device_ptr<type>        device_input(input);
+    common::device_ptr<output_type> device_output(size);
 
     // Running kernel
     constexpr unsigned int grid_size = (size / items_per_block);
     hipLaunchKernelGGL(
-        HIP_KERNEL_NAME(blocked_to_warp_striped_kernel<
-                type, output_type, items_per_block, items_per_thread
-        >),
-        dim3(grid_size), dim3(block_size), 0, 0,
-        device_input, device_output
-    );
+        HIP_KERNEL_NAME(
+            blocked_to_warp_striped_kernel<type, output_type, items_per_block, items_per_thread>),
+        dim3(grid_size),
+        dim3(block_size),
+        0,
+        0,
+        device_input.get(),
+        device_output.get());
     HIP_CHECK(hipGetLastError());
     HIP_CHECK(hipDeviceSynchronize());
 
     // Reading results
-    HIP_CHECK(
-        hipMemcpy(
-            output.data(), device_output,
-            output.size() * sizeof(typename decltype(output)::value_type),
-            hipMemcpyDeviceToHost
-        )
-    );
-
+    const auto output = device_output.load();
     ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output, expected));
-
-    HIP_CHECK(hipFree(device_input));
-    HIP_CHECK(hipFree(device_output));
 }
 
 template<class T,
@@ -480,7 +449,6 @@ auto test_block_exchange(int device_id) -> typename std::enable_if<Method == 3>:
     // Generate data
     std::vector<type> input(size);
     std::vector<output_type> expected(size);
-    std::vector<output_type> output(size, output_type(0));
 
     unsigned int current_device_warp_size;
     HIP_CHECK(::rocprim::host_warp_size(device_id, current_device_warp_size));
@@ -494,8 +462,8 @@ auto test_block_exchange(int device_id) -> typename std::enable_if<Method == 3>:
     test_utils::iota_modulo(values.begin(),
                             values.end(),
                             0,
-                            std::min<size_t>(test_utils::numeric_limits<type>::max(),
-                                             test_utils::numeric_limits<output_type>::max()));
+                            std::min<size_t>(rocprim::numeric_limits<type>::max(),
+                                             rocprim::numeric_limits<output_type>::max()));
     for(size_t bi = 0; bi < size / items_per_block; bi++)
     {
         for(size_t wi = 0; wi < warps_no; wi++)
@@ -518,42 +486,26 @@ auto test_block_exchange(int device_id) -> typename std::enable_if<Method == 3>:
     }
 
     // Preparing device
-    type* device_input;
-    HIP_CHECK(test_common_utils::hipMallocHelper(&device_input, input.size() * sizeof(typename decltype(input)::value_type)));
-    output_type* device_output;
-    HIP_CHECK(test_common_utils::hipMallocHelper(&device_output, output.size() * sizeof(typename decltype(output)::value_type)));
-
-    HIP_CHECK(
-        hipMemcpy(
-            device_input, input.data(),
-            input.size() * sizeof(type),
-            hipMemcpyHostToDevice
-        )
-    );
+    common::device_ptr<type>        device_input(input);
+    common::device_ptr<output_type> device_output(size);
 
     // Running kernel
     constexpr unsigned int grid_size = (size / items_per_block);
     hipLaunchKernelGGL(
-        HIP_KERNEL_NAME(warp_striped_to_blocked_kernel<type, output_type, items_per_block, items_per_thread>),
-        dim3(grid_size), dim3(block_size), 0, 0,
-        device_input, device_output
-    );
+        HIP_KERNEL_NAME(
+            warp_striped_to_blocked_kernel<type, output_type, items_per_block, items_per_thread>),
+        dim3(grid_size),
+        dim3(block_size),
+        0,
+        0,
+        device_input.get(),
+        device_output.get());
     HIP_CHECK(hipGetLastError());
     HIP_CHECK(hipDeviceSynchronize());
 
     // Reading results
-    HIP_CHECK(
-        hipMemcpy(
-            output.data(), device_output,
-            output.size() * sizeof(typename decltype(output)::value_type),
-            hipMemcpyDeviceToHost
-        )
-    );
-
+    const auto output = device_output.load();
     ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output, expected));
-
-    HIP_CHECK(hipFree(device_input));
-    HIP_CHECK(hipFree(device_output));
 }
 
 template<class T,
@@ -582,8 +534,7 @@ auto test_block_exchange(int /*device_id*/) -> typename std::enable_if<Method ==
 
     // Generate data
     std::vector<type> input(size);
-    std::vector<output_type> expected(size);
-    std::vector<output_type> output(size, output_type(0));
+    std::vector<output_type>  expected(size);
     std::vector<unsigned int> ranks(size);
 
     // Calculate input and expected results on host
@@ -597,8 +548,8 @@ auto test_block_exchange(int /*device_id*/) -> typename std::enable_if<Method ==
     test_utils::iota_modulo(values.begin(),
                             values.end(),
                             0,
-                            std::min<size_t>(test_utils::numeric_limits<type>::max(),
-                                             test_utils::numeric_limits<output_type>::max()));
+                            std::min<size_t>(rocprim::numeric_limits<type>::max(),
+                                             rocprim::numeric_limits<output_type>::max()));
     for(size_t bi = 0; bi < size / items_per_block; bi++)
     {
         for(size_t ti = 0; ti < block_size; ti++)
@@ -615,53 +566,28 @@ auto test_block_exchange(int /*device_id*/) -> typename std::enable_if<Method ==
     }
 
     // Preparing device
-    type* device_input;
-    HIP_CHECK(test_common_utils::hipMallocHelper(&device_input, input.size() * sizeof(typename decltype(input)::value_type)));
-    output_type* device_output;
-    HIP_CHECK(test_common_utils::hipMallocHelper(&device_output, output.size() * sizeof(typename decltype(output)::value_type)));
-    unsigned int* device_ranks;
-    HIP_CHECK(test_common_utils::hipMallocHelper(&device_ranks, ranks.size() * sizeof(typename decltype(ranks)::value_type)));
-
-    HIP_CHECK(
-        hipMemcpy(
-            device_input, input.data(),
-            input.size() * sizeof(type),
-            hipMemcpyHostToDevice
-        )
-    );
-
-    HIP_CHECK(
-        hipMemcpy(
-            device_ranks, ranks.data(),
-            ranks.size() * sizeof(unsigned int),
-            hipMemcpyHostToDevice
-        )
-    );
+    common::device_ptr<type>         device_input(input);
+    common::device_ptr<output_type>  device_output(size);
+    common::device_ptr<unsigned int> device_ranks(ranks);
 
     // Running kernel
     constexpr unsigned int grid_size = (size / items_per_block);
     hipLaunchKernelGGL(
-        HIP_KERNEL_NAME(scatter_to_blocked_kernel<type, output_type, items_per_block, items_per_thread>),
-        dim3(grid_size), dim3(block_size), 0, 0,
-        device_input, device_output, device_ranks
-    );
+        HIP_KERNEL_NAME(
+            scatter_to_blocked_kernel<type, output_type, items_per_block, items_per_thread>),
+        dim3(grid_size),
+        dim3(block_size),
+        0,
+        0,
+        device_input.get(),
+        device_output.get(),
+        device_ranks.get());
     HIP_CHECK(hipGetLastError());
     HIP_CHECK(hipDeviceSynchronize());
 
     // Reading results
-    HIP_CHECK(
-        hipMemcpy(
-            output.data(), device_output,
-            output.size() * sizeof(typename decltype(output)::value_type),
-            hipMemcpyDeviceToHost
-        )
-    );
-
+    const auto output = device_output.load();
     ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output, expected));
-
-    HIP_CHECK(hipFree(device_input));
-    HIP_CHECK(hipFree(device_output));
-    HIP_CHECK(hipFree(device_ranks));
 }
 
 template<class T,
@@ -690,8 +616,7 @@ auto test_block_exchange(int /*device_id*/) -> typename std::enable_if<Method ==
 
     // Generate data
     std::vector<type> input(size);
-    std::vector<output_type> expected(size);
-    std::vector<output_type> output(size, output_type(0));
+    std::vector<output_type>  expected(size);
     std::vector<unsigned int> ranks(size);
 
     // Calculate input and expected results on host
@@ -705,8 +630,8 @@ auto test_block_exchange(int /*device_id*/) -> typename std::enable_if<Method ==
     test_utils::iota_modulo(values.begin(),
                             values.end(),
                             0,
-                            std::min<size_t>(test_utils::numeric_limits<type>::max(),
-                                             test_utils::numeric_limits<output_type>::max()));
+                            std::min<size_t>(rocprim::numeric_limits<type>::max(),
+                                             rocprim::numeric_limits<output_type>::max()));
     for(size_t bi = 0; bi < size / items_per_block; bi++)
     {
         for(size_t ti = 0; ti < block_size; ti++)
@@ -725,53 +650,28 @@ auto test_block_exchange(int /*device_id*/) -> typename std::enable_if<Method ==
     }
 
     // Preparing device
-    type* device_input;
-    HIP_CHECK(test_common_utils::hipMallocHelper(&device_input, input.size() * sizeof(typename decltype(input)::value_type)));
-    output_type* device_output;
-    HIP_CHECK(test_common_utils::hipMallocHelper(&device_output, output.size() * sizeof(typename decltype(output)::value_type)));
-    unsigned int* device_ranks;
-    HIP_CHECK(test_common_utils::hipMallocHelper(&device_ranks, ranks.size() * sizeof(typename decltype(ranks)::value_type)));
-
-    HIP_CHECK(
-        hipMemcpy(
-            device_input, input.data(),
-            input.size() * sizeof(type),
-            hipMemcpyHostToDevice
-        )
-    );
-
-    HIP_CHECK(
-        hipMemcpy(
-            device_ranks, ranks.data(),
-            ranks.size() * sizeof(unsigned int),
-            hipMemcpyHostToDevice
-        )
-    );
+    common::device_ptr<type>         device_input(input);
+    common::device_ptr<output_type>  device_output(size);
+    common::device_ptr<unsigned int> device_ranks(ranks);
 
     // Running kernel
     constexpr unsigned int grid_size = (size / items_per_block);
     hipLaunchKernelGGL(
-        HIP_KERNEL_NAME(scatter_to_striped_kernel<type, output_type, items_per_block, items_per_thread>),
-        dim3(grid_size), dim3(block_size), 0, 0,
-        device_input, device_output, device_ranks
-    );
+        HIP_KERNEL_NAME(
+            scatter_to_striped_kernel<type, output_type, items_per_block, items_per_thread>),
+        dim3(grid_size),
+        dim3(block_size),
+        0,
+        0,
+        device_input.get(),
+        device_output.get(),
+        device_ranks.get());
     HIP_CHECK(hipGetLastError());
     HIP_CHECK(hipDeviceSynchronize());
 
     // Reading results
-    HIP_CHECK(
-        hipMemcpy(
-            output.data(), device_output,
-            output.size() * sizeof(typename decltype(output)::value_type),
-            hipMemcpyDeviceToHost
-        )
-    );
-
+    const auto output = device_output.load();
     ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output, expected));
-
-    HIP_CHECK(hipFree(device_input));
-    HIP_CHECK(hipFree(device_output));
-    HIP_CHECK(hipFree(device_ranks));
 }
 
 // Static for-loop
