@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -23,6 +23,21 @@
 #ifndef TEST_BLOCK_SCAN_KERNELS_HPP_
 #define TEST_BLOCK_SCAN_KERNELS_HPP_
 
+#include "../common_test_header.hpp"
+
+#include "../../common/utils_device_ptr.hpp"
+#include "test_utils.hpp"
+#include "test_utils_assertions.hpp"
+#include "test_utils_data_generation.hpp"
+#include "test_utils_types.hpp"
+
+#include <rocprim/block/block_scan.hpp>
+#include <rocprim/functional.hpp>
+
+#include <cstddef>
+#include <type_traits>
+#include <vector>
+
 template<
     int Method,
     unsigned int BlockSize,
@@ -175,6 +190,46 @@ void scan_kernel(T* device_output, T* device_output_b, T init)
     }
 }
 
+template<int                           Method,
+         unsigned int                  BlockSize,
+         rocprim::block_scan_algorithm Algorithm,
+         class T,
+         typename std::enable_if<Method == 6>::type* = nullptr>
+__global__
+__launch_bounds__(BlockSize)
+void scan_kernel(T* device_output, T* device_output_b, T init)
+{
+    (void)device_output_b;
+    const unsigned int                           index    = (blockIdx.x * BlockSize) + threadIdx.x;
+    T                                            input[1] = {device_output[index]};
+    T                                            output[1];
+    rocprim::block_scan<T, BlockSize, Algorithm> bscan;
+    bscan.inclusive_scan(input, init, output);
+    device_output[index] = output[0];
+}
+
+template<int                           Method,
+         unsigned int                  BlockSize,
+         rocprim::block_scan_algorithm Algorithm,
+         class T,
+         typename std::enable_if<Method == 7>::type* = nullptr>
+__global__
+__launch_bounds__(BlockSize)
+void scan_kernel(T* device_output, T* device_output_b, T init)
+{
+    const unsigned int                           index    = (blockIdx.x * BlockSize) + threadIdx.x;
+    T                                            input[1] = {device_output[index]};
+    T                                            output[1];
+    T                                            reduction;
+    rocprim::block_scan<T, BlockSize, Algorithm> bscan;
+    bscan.inclusive_scan(input, init, output, reduction);
+    device_output[index] = output[0];
+    if(threadIdx.x == 0)
+    {
+        device_output_b[blockIdx.x] = reduction;
+    }
+}
+
 template <
     class T,
     unsigned int BlockSize,
@@ -488,13 +543,13 @@ auto test_block_scan_input_arrays()
     SCOPED_TRACE(testing::Message() << "with size = " << size);
     SCOPED_TRACE(testing::Message() << "with grid_size = " << grid_size);
 
-    for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    for(size_t seed_index = 0; seed_index < number_of_runs; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
         SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
         // Generate data
-        std::vector<T> output = test_utils::get_random_data<T>(size, 2, 100, seed_value);
+        std::vector<T> output = test_utils::get_random_data_wrapped<T>(size, 2, 100, seed_value);
 
         // Calculate expected results on host
         std::vector<T> expected(output.size(), T(0));
@@ -509,39 +564,28 @@ auto test_block_scan_input_arrays()
         }
 
         // Writing to device memory
-        T* device_output;
-        HIP_CHECK(test_common_utils::hipMallocHelper(&device_output, output.size() * sizeof(typename decltype(output)::value_type)));
-
-        HIP_CHECK(
-            hipMemcpy(
-                device_output, output.data(),
-                output.size() * sizeof(T),
-                hipMemcpyHostToDevice
-            )
-        );
+        common::device_ptr<T> device_output(output);
 
         // Launching kernel
-        hipLaunchKernelGGL(
-            HIP_KERNEL_NAME(inclusive_scan_array_kernel<block_size, items_per_thread, algorithm, T, binary_op_type>),
-            dim3(grid_size), dim3(block_size), 0, 0,
-            device_output
-        );
+        hipLaunchKernelGGL(HIP_KERNEL_NAME(inclusive_scan_array_kernel<block_size,
+                                                                       items_per_thread,
+                                                                       algorithm,
+                                                                       T,
+                                                                       binary_op_type>),
+                           dim3(grid_size),
+                           dim3(block_size),
+                           0,
+                           0,
+                           device_output.get());
 
         HIP_CHECK(hipGetLastError());
         HIP_CHECK(hipDeviceSynchronize());
 
         // Read from device memory
-        HIP_CHECK(
-            hipMemcpy(
-                output.data(), device_output,
-                output.size() * sizeof(T),
-                hipMemcpyDeviceToHost
-            )
-        );
+        output = device_output.load();
 
         // Validating results
         ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output, expected));
-        HIP_CHECK(hipFree(device_output));
     }
 
 }
@@ -576,13 +620,13 @@ auto test_block_scan_input_arrays()
     SCOPED_TRACE(testing::Message() << "with size = " << size);
     SCOPED_TRACE(testing::Message() << "with grid_size = " << grid_size);
 
-    for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    for(size_t seed_index = 0; seed_index < number_of_runs; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
         SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
         // Generate data
-        std::vector<T> output = test_utils::get_random_data<T>(size, 2, 100, seed_value);
+        std::vector<T> output = test_utils::get_random_data_wrapped<T>(size, 2, 100, seed_value);
 
         // Output reduce results
         std::vector<T> output_reductions(size / block_size, T(0));
@@ -602,65 +646,32 @@ auto test_block_scan_input_arrays()
         }
 
         // Writing to device memory
-        T* device_output;
-        HIP_CHECK(test_common_utils::hipMallocHelper(&device_output, output.size() * sizeof(typename decltype(output)::value_type)));
-        T* device_output_reductions;
-        HIP_CHECK(
-            test_common_utils::hipMallocHelper(
-                &device_output_reductions,
-                output_reductions.size() * sizeof(typename decltype(output_reductions)::value_type)
-            )
-        );
-
-        HIP_CHECK(
-            hipMemcpy(
-                device_output, output.data(),
-                output.size() * sizeof(T),
-                hipMemcpyHostToDevice
-            )
-        );
-
-        HIP_CHECK(
-            hipMemcpy(
-                device_output_reductions, output_reductions.data(),
-                output_reductions.size() * sizeof(T),
-                hipMemcpyHostToDevice
-            )
-        );
+        common::device_ptr<T> device_output(output);
+        common::device_ptr<T> device_output_reductions(output_reductions);
 
         // Launching kernel
-        hipLaunchKernelGGL(
-            HIP_KERNEL_NAME(inclusive_scan_reduce_array_kernel<block_size, items_per_thread, algorithm, T, binary_op_type>),
-            dim3(grid_size), dim3(block_size), 0, 0,
-            device_output, device_output_reductions
-        );
+        hipLaunchKernelGGL(HIP_KERNEL_NAME(inclusive_scan_reduce_array_kernel<block_size,
+                                                                              items_per_thread,
+                                                                              algorithm,
+                                                                              T,
+                                                                              binary_op_type>),
+                           dim3(grid_size),
+                           dim3(block_size),
+                           0,
+                           0,
+                           device_output.get(),
+                           device_output_reductions.get());
 
         HIP_CHECK(hipGetLastError());
         HIP_CHECK(hipDeviceSynchronize());
 
         // Read from device memory
-        HIP_CHECK(
-            hipMemcpy(
-                output.data(), device_output,
-                output.size() * sizeof(T),
-                hipMemcpyDeviceToHost
-            )
-        );
-
-        HIP_CHECK(
-            hipMemcpy(
-                output_reductions.data(), device_output_reductions,
-                output_reductions.size() * sizeof(T),
-                hipMemcpyDeviceToHost
-            )
-        );
+        output            = device_output.load();
+        output_reductions = device_output_reductions.load();
 
         // Validating results
         ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output, expected));
         ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output_reductions, expected_reductions));
-
-        HIP_CHECK(hipFree(device_output));
-        HIP_CHECK(hipFree(device_output_reductions));
     }
 
 }
@@ -695,13 +706,13 @@ auto test_block_scan_input_arrays()
     SCOPED_TRACE(testing::Message() << "with size = " << size);
     SCOPED_TRACE(testing::Message() << "with grid_size = " << grid_size);
 
-    for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    for(size_t seed_index = 0; seed_index < number_of_runs; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
         SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
         // Generate data
-        std::vector<T> output = test_utils::get_random_data<T>(size, 2, 100, seed_value);
+        std::vector<T> output = test_utils::get_random_data_wrapped<T>(size, 2, 100, seed_value);
         std::vector<T> output_block_prefixes(size / items_per_block, T(0));
         T block_prefix = test_utils::get_random_value<T>(0, 100, seed_value);
 
@@ -721,67 +732,35 @@ auto test_block_scan_input_arrays()
         }
 
         // Writing to device memory
-        T* device_output;
-        HIP_CHECK(test_common_utils::hipMallocHelper(&device_output, output.size() * sizeof(typename decltype(output)::value_type)));
-        T* device_output_bp;
-        HIP_CHECK(
-            test_common_utils::hipMallocHelper(
-                &device_output_bp,
-                output_block_prefixes.size() * sizeof(typename decltype(output_block_prefixes)::value_type)
-            )
-        );
-
-        HIP_CHECK(
-            hipMemcpy(
-                device_output, output.data(),
-                output.size() * sizeof(T),
-                hipMemcpyHostToDevice
-            )
-        );
-
-        HIP_CHECK(
-            hipMemcpy(
-                device_output_bp, output_block_prefixes.data(),
-                output_block_prefixes.size() * sizeof(T),
-                hipMemcpyHostToDevice
-            )
-        );
+        common::device_ptr<T> device_output(output);
+        common::device_ptr<T> device_output_bp(output_block_prefixes);
 
         // Launching kernel
         hipLaunchKernelGGL(
-            HIP_KERNEL_NAME(
-                inclusive_scan_array_prefix_callback_kernel<block_size, items_per_thread, algorithm, T, binary_op_type>
-            ),
-            dim3(grid_size), dim3(block_size), 0, 0,
-            device_output, device_output_bp, block_prefix
-        );
+            HIP_KERNEL_NAME(inclusive_scan_array_prefix_callback_kernel<block_size,
+                                                                        items_per_thread,
+                                                                        algorithm,
+                                                                        T,
+                                                                        binary_op_type>),
+            dim3(grid_size),
+            dim3(block_size),
+            0,
+            0,
+            device_output.get(),
+            device_output_bp.get(),
+            block_prefix);
 
         HIP_CHECK(hipGetLastError());
         HIP_CHECK(hipDeviceSynchronize());
 
         // Read from device memory
-        HIP_CHECK(
-            hipMemcpy(
-                output.data(), device_output,
-                output.size() * sizeof(T),
-                hipMemcpyDeviceToHost
-            )
-        );
-
-        HIP_CHECK(
-            hipMemcpy(
-                output_block_prefixes.data(), device_output_bp,
-                output_block_prefixes.size() * sizeof(T),
-                hipMemcpyDeviceToHost
-            )
-        );
+        output                = device_output.load();
+        output_block_prefixes = device_output_bp.load();
 
         // Validating results
         ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output, expected));
-        ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output_block_prefixes, expected_block_prefixes));
-
-        HIP_CHECK(hipFree(device_output));
-        HIP_CHECK(hipFree(device_output_bp));
+        ASSERT_NO_FATAL_FAILURE(
+            test_utils::assert_eq(output_block_prefixes, expected_block_prefixes));
     }
 
 }
@@ -816,13 +795,13 @@ auto test_block_scan_input_arrays()
     SCOPED_TRACE(testing::Message() << "with size = " << size);
     SCOPED_TRACE(testing::Message() << "with grid_size = " << grid_size);
 
-    for(size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    for(size_t seed_index = 0; seed_index < number_of_runs; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
         SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
         // Generate data
-        std::vector<T> output = test_utils::get_random_data<T>(size, 2, 100, seed_value);
+        std::vector<T> output = test_utils::get_random_data_wrapped<T>(size, 2, 100, seed_value);
         const T init = test_utils::get_random_value<T>(0, 100, seed_value);
 
         // Calculate expected results on host
@@ -839,40 +818,29 @@ auto test_block_scan_input_arrays()
         }
 
         // Writing to device memory
-        T* device_output;
-        HIP_CHECK(test_common_utils::hipMallocHelper(&device_output, output.size() * sizeof(typename decltype(output)::value_type)));
-
-        HIP_CHECK(
-            hipMemcpy(
-                device_output, output.data(),
-                output.size() * sizeof(T),
-                hipMemcpyHostToDevice
-            )
-        );
+        common::device_ptr<T> device_output(output);
 
         // Launching kernel
-        hipLaunchKernelGGL(
-            HIP_KERNEL_NAME(exclusive_scan_array_kernel<block_size, items_per_thread, algorithm, T, binary_op_type>),
-            dim3(grid_size), dim3(block_size), 0, 0,
-            device_output, init
-        );
+        hipLaunchKernelGGL(HIP_KERNEL_NAME(exclusive_scan_array_kernel<block_size,
+                                                                       items_per_thread,
+                                                                       algorithm,
+                                                                       T,
+                                                                       binary_op_type>),
+                           dim3(grid_size),
+                           dim3(block_size),
+                           0,
+                           0,
+                           device_output.get(),
+                           init);
 
         HIP_CHECK(hipGetLastError());
         HIP_CHECK(hipDeviceSynchronize());
 
         // Read from device memory
-        HIP_CHECK(
-            hipMemcpy(
-                output.data(), device_output,
-                output.size() * sizeof(T),
-                hipMemcpyDeviceToHost
-            )
-        );
+        output = device_output.load();
 
         // Validating results
         ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output, expected));
-
-        HIP_CHECK(hipFree(device_output));
     }
 
 }
@@ -907,13 +875,13 @@ auto test_block_scan_input_arrays()
     SCOPED_TRACE(testing::Message() << "with size = " << size);
     SCOPED_TRACE(testing::Message() << "with grid_size = " << grid_size);
 
-    for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    for(size_t seed_index = 0; seed_index < number_of_runs; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
         SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
         // Generate data
-        std::vector<T> output = test_utils::get_random_data<T>(size, 2, 100, seed_value);
+        std::vector<T> output = test_utils::get_random_data_wrapped<T>(size, 2, 100, seed_value);
 
         // Output reduce results
         std::vector<T> output_reductions(size / items_per_block);
@@ -939,58 +907,33 @@ auto test_block_scan_input_arrays()
         }
 
         // Writing to device memory
-        T* device_output;
-        HIP_CHECK(test_common_utils::hipMallocHelper(&device_output, output.size() * sizeof(typename decltype(output)::value_type)));
-        T* device_output_reductions;
-        HIP_CHECK(
-            test_common_utils::hipMallocHelper(
-                &device_output_reductions,
-                output_reductions.size() * sizeof(typename decltype(output_reductions)::value_type)
-            )
-        );
-
-        HIP_CHECK(
-            hipMemcpy(
-                device_output, output.data(),
-                output.size() * sizeof(T),
-                hipMemcpyHostToDevice
-            )
-        );
+        common::device_ptr<T> device_output(output);
+        common::device_ptr<T> device_output_reductions(output_reductions.size());
 
         // Launching kernel
-        hipLaunchKernelGGL(
-            HIP_KERNEL_NAME(
-                exclusive_scan_reduce_array_kernel<block_size, items_per_thread, algorithm, T, binary_op_type>
-            ),
-            dim3(grid_size), dim3(block_size), 0, 0,
-            device_output, device_output_reductions, init
-        );
+        hipLaunchKernelGGL(HIP_KERNEL_NAME(exclusive_scan_reduce_array_kernel<block_size,
+                                                                              items_per_thread,
+                                                                              algorithm,
+                                                                              T,
+                                                                              binary_op_type>),
+                           dim3(grid_size),
+                           dim3(block_size),
+                           0,
+                           0,
+                           device_output.get(),
+                           device_output_reductions.get(),
+                           init);
 
         HIP_CHECK(hipGetLastError());
         HIP_CHECK(hipDeviceSynchronize());
 
         // Read from device memory
-        HIP_CHECK(
-            hipMemcpy(
-                output.data(), device_output,
-                output.size() * sizeof(T),
-                hipMemcpyDeviceToHost
-            )
-        );
-
-        HIP_CHECK(
-            hipMemcpy(
-                output_reductions.data(), device_output_reductions,
-                output_reductions.size() * sizeof(T),
-                hipMemcpyDeviceToHost
-            )
-        );
+        output            = device_output.load();
+        output_reductions = device_output_reductions.load();
 
         // Validating results
         ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output, expected));
         ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output_reductions, expected_reductions));
-        HIP_CHECK(hipFree(device_output));
-        HIP_CHECK(hipFree(device_output_reductions));
     }
 
 }
@@ -1025,13 +968,13 @@ auto test_block_scan_input_arrays()
     SCOPED_TRACE(testing::Message() << "with size = " << size);
     SCOPED_TRACE(testing::Message() << "with grid_size = " << grid_size);
 
-    for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    for(size_t seed_index = 0; seed_index < number_of_runs; seed_index++)
     {
         unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
         SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
 
         // Generate data
-        std::vector<T> output = test_utils::get_random_data<T>(size, 2, 100, seed_value);
+        std::vector<T> output = test_utils::get_random_data_wrapped<T>(size, 2, 100, seed_value);
         std::vector<T> output_block_prefixes(size / items_per_block);
         T block_prefix = test_utils::get_random_value<T>(0, 100, seed_value);
 
@@ -1056,59 +999,35 @@ auto test_block_scan_input_arrays()
         }
 
         // Writing to device memory
-        T* device_output;
-        HIP_CHECK(test_common_utils::hipMallocHelper(&device_output, output.size() * sizeof(typename decltype(output)::value_type)));
-        T* device_output_bp;
-        HIP_CHECK(
-            test_common_utils::hipMallocHelper(
-                &device_output_bp,
-                output_block_prefixes.size() * sizeof(typename decltype(output_block_prefixes)::value_type)
-            )
-        );
-
-        HIP_CHECK(
-            hipMemcpy(
-                device_output, output.data(),
-                output.size() * sizeof(T),
-                hipMemcpyHostToDevice
-            )
-        );
+        common::device_ptr<T> device_output(output);
+        common::device_ptr<T> device_output_bp(output_block_prefixes.size());
 
         // Launching kernel
         hipLaunchKernelGGL(
-            HIP_KERNEL_NAME(
-                exclusive_scan_prefix_callback_array_kernel<block_size, items_per_thread, algorithm, T, binary_op_type>
-            ),
-            dim3(grid_size), dim3(block_size), 0, 0,
-            device_output, device_output_bp, block_prefix
-        );
+            HIP_KERNEL_NAME(exclusive_scan_prefix_callback_array_kernel<block_size,
+                                                                        items_per_thread,
+                                                                        algorithm,
+                                                                        T,
+                                                                        binary_op_type>),
+            dim3(grid_size),
+            dim3(block_size),
+            0,
+            0,
+            device_output.get(),
+            device_output_bp.get(),
+            block_prefix);
 
         HIP_CHECK(hipGetLastError());
         HIP_CHECK(hipDeviceSynchronize());
 
         // Read from device memory
-        HIP_CHECK(
-            hipMemcpy(
-                output.data(), device_output,
-                output.size() * sizeof(T),
-                hipMemcpyDeviceToHost
-            )
-        );
-
-        HIP_CHECK(
-            hipMemcpy(
-                output_block_prefixes.data(), device_output_bp,
-                output_block_prefixes.size() * sizeof(T),
-                hipMemcpyDeviceToHost
-            )
-        );
+        output                = device_output.load();
+        output_block_prefixes = device_output_bp.load();
 
         // Validating results
         ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output, expected));
-        ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output_block_prefixes, expected_block_prefixes));
-
-        HIP_CHECK(hipFree(device_output));
-        HIP_CHECK(hipFree(device_output_bp));
+        ASSERT_NO_FATAL_FAILURE(
+            test_utils::assert_eq(output_block_prefixes, expected_block_prefixes));
     }
 
 }
